/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "Client.hpp"

Client::Client(GraphicSet* gs, BasicUserInterface* obui,
               Map* omap, UnitContainer* ounits, PLAYER oplyr){
    bui = obui;
    map = omap;
    units = ounits;
    plyr = oplyr;

    ui_engine = new UI_Engine(bui->getSize(),Coordinates(0,0),gs,bui);
    engine = new Engine(map, units, ui_engine, plyr);

    // add ClientMainMenu
    uim = new UI_Menu(Coordinates(128, 196), Coordinates(0,16), gs, ui_engine);
    cmm = new ClientMainMenu( uim, units, plyr);
    engine->addChild(cmm);

    pointerpos = new Coordinates();
    oldpointerpos = new Coordinates();

    selectedunit = units->selectNextUnit(MAGICIAN, plyr);
    engine->center(selectedunit->getPos());
}

Client::~Client(){
    delete engine;
    delete ui_engine;
    delete cmm;
    delete pointerpos;
    delete oldpointerpos;
}

RUNMODE Client::run(){
    INPUTTYPE input = NOINPUT;
    while (input != QUIT){
        input = clientLoop();

        /* will need to refresh map and
         * units here if RunMode should be
         * run with a Server */
    }
    return LOBBY;
}

INPUTTYPE Client::clientLoop(){
    cmm->refresh();
    cmm->print();

    /* check if the player has lost
     * by checking if his magician exists */
    if (!units->checkForUnit(MAGICIAN, plyr)){

        /* switch to PLAYER0 so the player cannot play
         * anymore */
        plyr = PLAYER0;
        engine->switchPlayer(plyr);

        /* replace menu by a notification that the
         * player has lost the game */
        delete cmm;
        cmm = new PlayerLooseMenu(uim);
        cmm->refresh();
        cmm->print();
    }

    engine->refresh();

    // clientloop
    bool clientloop = true;
    while (clientloop){
        engine->print();
        currinput = bui->getInput(NOINPUT);
        if(currinput == QUIT){
            return QUIT;
        }
        switch(engine->getInput(currinput)){
        case QUIT:
            return QUIT;
        case ENDROUND:
            clientloop = false;
            break;
        case SELECT:
            selectedfield = engine->getSelectedField();
            if ( units->selectUnit(selectedfield) != NULL &&
                 !(units->selectUnit(selectedfield)->isAlive())){
                units->selectUnit(Coordinates());
            }
            engine->refresh();
            break;
        case SELECTUNIT:
            selectedunit = units->getSelectedUnit();
            cmm->print();
            engine->center(selectedunit->getPos());
            engine->refresh();
            break;
        case RMB:
            selectedfield = engine->getSelectedField();
            selectedunit = units->getSelectedUnit();
            if (selectedunit != NULL && selectedunit->getOwner() == plyr){
                (units->getSelectedUnit())->command(selectedfield);
                engine->refresh();
            }
            break;
        case MOVEMENU:
            *oldpointerpos = bui->getPointerPos();
            while (bui->getInput(NOINPUT) == LMBHOLD){
                *pointerpos = bui->getPointerPos();
                if (*pointerpos == Coordinates(-1,-1)){
                    *pointerpos = *oldpointerpos;
                } else {
                    cmm->move(*pointerpos - *oldpointerpos);
                    cmm->print();
                }
                *oldpointerpos = *pointerpos;
            }
            break;
        case NOINPUT:
            break;
        default:
            break;
        }
    // end of clientloop
    }
    return ENDROUND;
}
