/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __CLIENT_H__
#define	__CLIENT_H__



#include <vector>
#include <iostream>

#include "../shared/definitions/Map.hpp"
#include "../shared/definitions/Coordinates.hpp"
#include "../shared/units/UnitContainer.hpp"
#include "../shared/misc/MapLoader.hpp"
#include "../shared/gui/engine/Engine.hpp"
#include "../shared/gui/allegro/BasicUserInterface.hpp"
#include "../shared/RunMode.hpp"
#include "gui/ClientMainMenu.hpp"
#include "gui/PlayerLooseMenu.hpp"


class Client : public RunMode {
public:
    Client(GraphicSet* ogs, BasicUserInterface* obui,
           Map* omap, UnitContainer* ounits, PLAYER plyrnum);
    ~Client();

    RUNMODE run();

    /** the clientLoop runs until the player ends
     * his round or wants to quit */
    INPUTTYPE clientLoop();

private:
    /** the map contains the terrain information
     * the player currently has */
    Map* map;

    /** the UnitContainer should contain (pointers
     * to) all units known by the player at the time
     * he saw them */
    UnitContainer* units;

    /** this UI_Engine is addicted to one player,
     * it only contains information he might know
     */
    UI_Engine* ui_engine;

    UI_Menu* uim;
    Engine* engine;

    Coordinates selectedfield;
    Unit* selectedunit;

    BasicUserInterface* bui;
    INPUTTYPE currinput;

    Menu* cmm;

    /** used to move menus around */
    Coordinates* pointerpos;
    Coordinates* oldpointerpos;

    // which player is controlled by this client?
    PLAYER plyr;
};

#endif	/* __CLIENT_H__ */
