/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "Editor.hpp"

Editor::Editor(GraphicSet* ogs, BasicUserInterface* obui, FSHandler* ofsh){
    fsh = ofsh;

    /* Initializing a MapLoader will trigger loading a
     * map. This might end in a failure, so let us just
     * try it. */
    try{
        ml = new MapLoader((char*) "maps/Editor.map", fsh);
    }
    catch(Hexception& e){
        std::cerr << e.what();
    }
    map = ml->loadedMap();
    player = ml->loadedUnits();
    delete ml;
    ml = NULL;

    gs = ogs;
    bui = obui;

    ui_engine = new UI_Engine(bui->getSize(),Coordinates(0,0),gs,bui);
    engine = new Engine(map, player, ui_engine, PLAYER0);

    // add EditorMainMenu
    UI_Menu* uim = new UI_Menu(Coordinates(128, 196), Coordinates(0,16), gs, ui_engine);
    emm = new EditorMainMenu(uim);
    engine->addChild(emm);

    pointerpos = new Coordinates();
    oldpointerpos = new Coordinates();
}

Editor::~Editor(){
    delete engine;
    delete map;
    delete ui_engine;
    delete player;
    delete emm;
    delete pointerpos;
    delete oldpointerpos;
}

RUNMODE Editor::run(){
    emm->refresh();
    emm->print();
    engine->refresh();
    bool endgame = false;
    while (!endgame){
        engine->print();
        currinput = bui->getInput(NOINPUT);
        if(currinput == QUIT){
            endgame = true;
        }
        switch(engine->getInput(currinput)){
        case QUIT:
            endgame = true;
            break;
        case SCROLL:
            engine->refresh();
            break;
        case LMB:
            editedfield = engine->getSelectedField();
            map->setHeight(editedfield, 1);
            engine->refresh();
            break;
        case LMBHOLD:
            if(editedfield != engine->getSelectedField()){
                editedfield = engine->getSelectedField();
                map->setHeight(editedfield, 1);
                engine->refresh();
            }
            break;
        case RMB:
            editedfield = engine->getSelectedField();
            map->setHeight(editedfield, 0);
            engine->refresh();
            break;
        case RMBHOLD:
            if(editedfield != engine->getSelectedField()){
                editedfield = engine->getSelectedField();
                map->setHeight(editedfield, 0);
                engine->refresh();
            }
            break;
        case SAVE:
            saveMapToDisk((char*) "maps/Editor.map");
            break;
        case MOVEMENU:
            *oldpointerpos = bui->getPointerPos();
            while (bui->getInput(NOINPUT) == LMBHOLD){
                *pointerpos = bui->getPointerPos();
                if (*pointerpos == Coordinates(-1,-1)){
                    *pointerpos = *oldpointerpos;
                }
                else{
                    emm->move(*pointerpos - *oldpointerpos);
                    emm->refresh();
                    emm->print();
                }
                *oldpointerpos = *pointerpos;
            }
            break;
        case NOINPUT:
            editedfield = Coordinates(-1,-1);
            break;
        default:
            break;
        }
    }
    return LOBBY;
}

void Editor::saveMapToDisk(char* filename) {
    std::ofstream ofs;
    ofs.open(filename, std::fstream::trunc);
    ofs << "HWmap_V01 " << map->getSize().x << " " << map->getSize().y << std::endl;
    for (int y=0; y < map->getSize().y; y++) {
        for (int x=0; x < map->getSize().x; x++) {
            switch(map->getTerrain(Coordinates(x,y))) {
            case WATER:
                ofs << "00";
                break;
            case GRASS:
                ofs << "01";
                break;
            default:
                ofs << "00";
                break;
            }
            if (x < map->getSize().x - 1)
                ofs << " ";
        }
        ofs << std::endl;
    }
    ofs.close();
}
