/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* The Editor will create a blank 
 * map with no units placed on it. 
 * When run it will be controlled
 * via its own user interface.*/

#ifndef __EDITOR_H__
#define __EDITOR_H__

#include <string>
#include <fstream>

#include "../shared/gui/allegro/BasicUserInterface.hpp"
#include "../shared/definitions/Alias.hpp"
#include "../shared/units/UnitContainer.hpp"
#include "../shared/RunMode.hpp"
#include "../shared/definitions/Map.hpp"
#include "../shared/misc/MapLoader.hpp"
#include "../shared/misc/FSHandler.hpp"
#include "../shared/definitions/Coordinates.hpp"
#include "../shared/gui/engine/Engine.hpp"
#include "gui/EditorMainMenu.hpp"

class Editor : public RunMode{
public:
    Editor(GraphicSet* ogs, BasicUserInterface* obui, FSHandler* ofsh);
    ~Editor();

    RUNMODE run();
    // this runs the editor
    void saveMapToDisk(char* filename);
private:
    FSHandler* fsh;

    Engine* engine;
    /* This will be used to display
     * the user interface, the map
     * and all units. */

    UI_Engine* ui_engine;

    Map* map;
    /* The map that is edited.*/

    MapLoader* ml;

    UnitContainer* player;
    /* Units will be saved separately
     * for every player in his own
     * field in this vector. */

    GraphicSet* gs;
    BasicUserInterface* bui;

    Coordinates editedfield;
    /* saves which field was modified */

    INPUTTYPE currinput;

    EditorMainMenu* emm;

    Coordinates* pointerpos;
    Coordinates* oldpointerpos;
};

#endif /* __EDITOR_H__ */
