/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "HotSeat.hpp"

HotSeat::HotSeat(GraphicSet* ogs, BasicUserInterface* obui, FSHandler* ofsh){
    gs = ogs;
    bui = obui;
    fsh = ofsh;

    // init graphical elements
    // add PlayerSwitchMenu
    uim = new UI_Menu(bui->getSize(),Coordinates(0,0),ogs,bui);
    sgm = new StartGameMenu(uim);
    psm = NULL;
}

HotSeat::~HotSeat(){
    units.clear();
}

void HotSeat::startGame(std::string level){
    // init logical elements
    server = new Server(level, fsh);

    map.push_back(server->getMap(PLAYER1));
    map.push_back(server->getMap(PLAYER2));
    units.push_back(new UnitContainer(*(server->getUnits(PLAYER1))));
    units.push_back(new UnitContainer(*(server->getUnits(PLAYER2))));
    client.push_back(new Client( gs, bui, map[0], units[0], PLAYER1));
    client.push_back(new Client( gs, bui, map[1], units[1], PLAYER2));
}

RUNMODE HotSeat::run(){
    bool loop = true;
    // loop for selecting game options
    sgm->refresh();
    while (loop){
        sgm->print();
        currinput = bui->getInput(NOINPUT);
        if(currinput == QUIT){
            return LOBBY;
        }
        switch(sgm->getInput(currinput)){
        case QUIT:
            return LOBBY;
        case START:
            loop = false;
            break;
        default:
            break;
        }
    }

    // start the game
    startGame(sgm->getSelectedMap());
    delete sgm;
    sgm = NULL;
    psm = new PlayerSwitchMenu(uim);
    psm->refresh();

    loop = true;
    // loop of hotseat game
    while (loop) {
        psm->print();
        currinput = bui->getInput(NOINPUT);
        if(currinput == QUIT){
            return LOBBY;
        }
        switch(psm->getInput(currinput)){
        case QUIT:
            return LOBBY;
        case SWITCHPLAYER:
            psm->refresh();
            break;
        case SWITCH2P1:
            client[0]->clientLoop();
            psm->refresh();
            break;
        case SWITCH2P2:
            client[1]->clientLoop();
            psm->refresh();
            break;
        case ENDROUND:
            server->updateUnits(units[0], PLAYER1);
            server->updateUnits(units[1], PLAYER2);
            server->doMovement();
            units[0]->sync(server->getUnits(PLAYER1));
            units[1]->sync(server->getUnits(PLAYER2));
            //HACK: server ought to have a nice tidy method
            server->getUnits(PLAYER2)->tidy();


            if (!server->getUnits(PLAYER2)->checkForUnit(MAGICIAN, PLAYER2)){
                psm->setWinner(1);
            } else if (!server->getUnits(PLAYER2)->checkForUnit(MAGICIAN, PLAYER1)){
                psm->setWinner(2);
            }
            psm->refresh();
            break;
        default:
            break;
        }
    // end of loop of hotseat game
    }
    return LOBBY;
}
