/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "Lobby.hpp"

Lobby::Lobby(GraphicSet* gs, BasicUserInterface* bui){
    basicuserinterface = bui;
    graphicset = gs;
    ui_menu = new UI_Menu(bui->getSize(),Coordinates(0,0),graphicset,basicuserinterface);
    menu = new MainMenu(ui_menu, this);
}

Lobby::~Lobby(){
	delete menu;
	delete ui_menu;
}

RUNMODE Lobby::run(){
    nextrunmode = LOBBY;
    menu->refresh();
    while (nextrunmode == LOBBY){
        currinput = basicuserinterface->getInput(NOINPUT);
        menu->print();
        switch(menu->getInput(currinput)){
        case SCROLL:
            break;
        case SELECT:
            menu->refresh();
            break;
        case QUIT:
            menu = new QuitMenu(ui_menu, this);
            menu->refresh();
            break;
        default:
            break;
        }
    }
    return nextrunmode;
}

void Lobby::setNextRunMode(RUNMODE nrm){
	nextrunmode = nrm;
}

void Lobby::setGraphicSet(std::string gs){
	graphicset->setGraphicSet(gs);
}

void Lobby::changeMenu(MENUTYPE menutype){
	delete menu;
	switch(menutype){
	case CONFIGMENU:
		menu = new ConfigMenu(ui_menu, this, basicuserinterface);
		break;
	case MAINMENU:
		menu = new MainMenu(ui_menu, this);
		break;
	case QUITMENU:
		menu = new QuitMenu(ui_menu, this);
		break;
	default:
		menu = NULL;
		break;
	}
    menu->refresh();
}

