/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef _SERVER_H
#define	_SERVER_H

#include <vector>

#include "../shared/definitions/Alias.hpp"
#include "../shared/misc/MapLoader.hpp"
#include "../shared/units/UnitContainer.hpp"
#include "../shared/misc/FSHandler.hpp"
#include "../client/Client.hpp"

class Server{
public:
    /** You just have to tell the server which
     * level to load. */
    Server(std::string level, FSHandler* ofsh);

    /** currently all players get the same Map */
    Map* getMap(PLAYER plyr);

    /** currently all UnitContainers contain pointers
     * to the same Units, but selection may differ */
    UnitContainer* getUnits(PLAYER plyr);

    void doMovement();

    void updateUnits(UnitContainer* orig, PLAYER plyr);
private:
    FSHandler* fsh;

    /** map [0] is the fully visible map with every
     * information up to date, map[n>0] is for PLAYERn*/
    std::vector<Map*> map;

    /** units[0] means ALL units (the master copy),
     * units[n>0] is for PLAYERn */
    std::vector<UnitContainer*> units;

    MapLoader* ml;
};

#endif	/* _SERVER_H */
