/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "Map.hpp"

Map::Map(unsigned int x, unsigned int y, int h){
    size.x = x;
    size.y = y;
    content = new int*[x];
    for (unsigned int i=0; i<x; i++){
        content[i] = new int[y];
    }
    for (unsigned int i=0; i < x; i++) {
        for (unsigned int j=0; j < y; j++) {
            setHeight(i,j,h);
        }
    }
    waterlevel = h;
}

Map::~Map(){
    for (int i=0; i < size.x; i++){
	    delete [] content[i];
    }
    delete content;
}

Map::Map(Map& original){
	size = original.size;
	if(size.x <0 || size.y <0){
		throw "Invalid map size.";
	}
    content = new int*[size.x];
    for (int i=0; i < size.x; i++){
	    content[i] = new int[size.y];
    }
    waterlevel = original.waterlevel;
    for (int x=0; x < size.x; x++){
    	for (int y=0; y < size.x; y++){
	    	content[x][y] = original.content[x][y];
    	}
    }
}

Coordinates Map::getSize(){
	return size;
}

int Map::getHeight(unsigned int x, unsigned int y){
	return content[x][y];
}
int Map::getHeight(Coordinates f){
	return content[f.x][f.y];
}

void Map::setHeight(unsigned int x, unsigned int y, int h){
	setHeight(Coordinates(x,y), h);
}

void Map::setHeight(Coordinates pos, int h){
	if (!(pos < Coordinates(0,0)) && (pos < size)){
    	content[pos.x][pos.y] = h;
	}
}

int Map::getWaterLevel(){
    return waterlevel;
}
TERRAINTYPE Map::getTerrain(Coordinates pos){
	if ((pos > Coordinates(-1,-1)) && (pos < size)){
    	if(waterlevel > content[pos.x][pos.y]){
    	    return WATER;
    	}else{
    	    return GRASS;
    	}
	}
	return WATER;
}

