/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* The class 'Map' contains the maps the
 * the game is played on. The information 
 * that need to be saved are the size of 
 * the map, a heightmap and the height of the
 * water in the level. */

#ifndef _MAP_H
#define	_MAP_H


typedef unsigned int TERRAINTYPE;
enum Terrains{
        WATER =0, GRASS =1, BLACK
};

#include <exception>

#include "Alias.hpp"
#include "Coordinates.hpp"

class Map{
private:
    Coordinates size;
    // the size of the Map
    
    int** content;
    // this will contain a heightmap when size is specified
    
    int waterlevel;
    /* This specifies the "mean sea level" of the Map.
     * It may be used to be able to change it while 
     * playing or to customize standard maps in an easy
     * way. */
public:
    ~Map();
	/* deletes the Map and frees all memory that
	 * was requested by this object */
	
    Map(unsigned int x, unsigned  int y, int h);
	/* creates a map of x*y fields, h specifies
	 * the water level */
	
    Map(Map&);
    // creates a new Map as a copy of another one
    
    Coordinates getSize();
    // returns the size of the Map
    
    int getHeight(unsigned int x, unsigned int y);
    int getHeight(Coordinates);
    // returns the height of the specified Coordinates
    
    void setHeight(unsigned int x, unsigned int y, int h);
    void setHeight(Coordinates, int h);
    // lets you set the height of one field.
    
    int getWaterLevel();
    // returns the waterlevel
    
    TERRAINTYPE getTerrain(Coordinates);
};

#endif	/* _MAP_H */
