/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "GraphicSet.hpp"


const char* GraphicSet::c_filename(boost::filesystem::path path, boost::filesystem::path file){
    path /= file;
    return (path.file_string()).c_str();
}

GraphicSet::GraphicSet(std::string ographicsdir, FSHandler* ofsh){
    fsh = ofsh;
    graphicsdir = "graphics/allegro/";
    graphicsdir /= ographicsdir;
    graphicsdir = fsh->getFullPath(graphicsdir);
    // load the set of graphics
    setNull();
    loadGraphics();
}

void GraphicSet::setGraphicSet(std::string ographicsdir){
    graphicsdir = "graphics/allegro/";
    graphicsdir /= ographicsdir;
    /* need to unload and reload since graphics
     * where already loaded by constructor
     */
    unLoadGraphics();
    setNull();
    loadGraphics();
}

GraphicSet::~GraphicSet(){
    unLoadGraphics();
}

void GraphicSet::loadGraphics(){
    grass1        = load_bitmap(c_filename(graphicsdir,"/grass1.tga"), NULL);
    water1        = load_bitmap(c_filename(graphicsdir,"/water1.tga"), NULL);
    fog           = load_bitmap(c_filename(graphicsdir,"/fog.tga"), NULL);
    fogtile       = load_bitmap(c_filename(graphicsdir,"/tile_fog.tga"), NULL);
    pointer[0]    = load_bitmap(c_filename(graphicsdir,"/pointer.tga"), NULL);
    pointer[1]    = load_bitmap(c_filename(graphicsdir,"/pointer1.tga"), NULL);
    pointer[2]    = load_bitmap(c_filename(graphicsdir,"/pointer2.tga"), NULL);
    marker        = load_bitmap(c_filename(graphicsdir,"/marker.tga"), NULL);
    magician[1]   = load_bitmap(c_filename(graphicsdir,"/magician0.tga"), NULL);
    magician[1]   = load_bitmap(c_filename(graphicsdir,"/magician1.tga"), NULL);
    magician[2]   = load_bitmap(c_filename(graphicsdir,"/magician2.tga"), NULL);
    warrior[0]    = load_bitmap(c_filename(graphicsdir,"/warrior0.tga"), NULL);
    warrior[1]    = load_bitmap(c_filename(graphicsdir,"/warrior1.tga"), NULL);
    warrior[2]    = load_bitmap(c_filename(graphicsdir,"/warrior2.tga"), NULL);
    scout[0]      = load_bitmap(c_filename(graphicsdir,"/scout0.tga"), NULL);
    scout[1]      = load_bitmap(c_filename(graphicsdir,"/scout1.tga"), NULL);
    scout[2]      = load_bitmap(c_filename(graphicsdir,"/scout2.tga"), NULL);
    terraf[0]     = load_bitmap(c_filename(graphicsdir,"/terraf0.tga"), NULL);
    terraf[1]     = load_bitmap(c_filename(graphicsdir,"/terraf1.tga"), NULL);
    terraf[2]     = load_bitmap(c_filename(graphicsdir,"/terraf2.tga"), NULL);
}

void GraphicSet::unLoadGraphics(){
    destroy_bitmap(grass1);
    destroy_bitmap(water1);
    destroy_bitmap(marker);
    destroy_bitmap(fog);
    destroy_bitmap(fogtile);
    for(int i=0; i<=2; i++){
        destroy_bitmap(pointer[i]);
        destroy_bitmap(magician[i]);
        destroy_bitmap(warrior[i]);
        destroy_bitmap(scout[i]);
        destroy_bitmap(terraf[i]);
    }
}

void GraphicSet::setNull(){
    grass1 = NULL;
    water1 = NULL;
    marker = NULL;
    fog = NULL;
    fogtile = NULL;
    for(int i=0; i<=2; i++){
        pointer[i] = NULL;
        magician[i] = NULL;
        warrior[i] = NULL;
        scout[i] = NULL;
        terraf[i] = NULL;
    }
}
