/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* The GraphicSet provides all the graphics
 * needed by the UI_Engine to display units,
 * terrains and other stuff. */

#ifndef __GRAPHICSET__H__
#define __GRAPHICSET__H__

#include <allegro.h>
#include <string>
#include <vector>
#include <boost/filesystem.hpp>
#include <boost/filesystem/fstream.hpp>

#include "../../definitions/Alias.hpp"
#include "../../misc/FSHandler.hpp"

class GraphicSet{
public:
    GraphicSet(std::string, FSHandler*);
    ~GraphicSet();

    /// lets you choose a new(!) GraphicSet
    void setGraphicSet(std::string);
    
    // bitmaps needed to display terrains
    BITMAP* grass1;
    BITMAP* water1;
    
    // used to highlight selected units
    BITMAP* marker;

    BITMAP* fogtile;
    BITMAP* fog;
    
    // bitmaps for the pointer
    BITMAP* pointer[3];
    
    // bitmaps needed to display units
    BITMAP* magician[3];
    BITMAP* warrior[3];
    BITMAP* scout[3];
    BITMAP* terraf[3];

private:
    FSHandler* fsh;

    // will return a filename handable by allegro
    const char* c_filename(boost::filesystem::path path, boost::filesystem::path file);

    // saves which graphicset is currently used
    boost::filesystem::path graphicsdir;

    void loadGraphics();
    // loads the graphics

    // unloads the graphics
    void unLoadGraphics();

    /** sets all pointers to NULL */
    void setNull();
};

#endif /* __GRAPHICSET__H__ */
