/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "UI_Engine.hpp"

UI_Engine::UI_Engine(Coordinates size, Coordinates pos, GraphicSet* gs, UI* par){
    init(size, pos, gs, par);
    tilesize.x = (graphicset->water1)->w;
    tilesize.y = (graphicset->water1)->h;
    unitsize.x = (graphicset->warrior[1])->w;
    unitsize.y = (graphicset->warrior[1])->h;
}

UI_Engine::~UI_Engine(){
    ;
}

void UI_Engine::drawLine(Coordinates from, Coordinates to, int col){
    fastline( buffer, from.x, from.y, to.x, to.y,
          makecol(col/0x10000,col/0x100%0x100,col%0x100));
}

void UI_Engine::drawLine(std::vector<Coordinates> coos, int col){
    if (coos.size() >= 2){
        for (unsigned int i=0; i<coos.size()-1 ; i++){
            drawLine(coos[i], coos[i+1], col);
        }
    }
}

void UI_Engine::drawBlack(){
    clear_bitmap(buffer);
}

void UI_Engine::drawTerrain(TERRAINTYPE type, Coordinates pos){
    switch(type){
    case BLACK:
        masked_blit( graphicset->fogtile,buffer,0,0,pos.x,pos.y,tilesize.x,tilesize.y);
        break;
    case WATER:
        masked_blit( graphicset->water1,buffer,0,0,pos.x,pos.y,tilesize.x,tilesize.y);
        break;
    case GRASS:
        masked_blit( graphicset->grass1,buffer,0,0,pos.x,pos.y,tilesize.x,tilesize.y);
        break;
    default:
        break;
    }
}

void UI_Engine::drawUnit(UNITTYPE type, PLAYER owner, Coordinates pos){
    switch(type){
    case WARRIOR:
        masked_blit( graphicset->warrior[owner],buffer,0,0,pos.x-unitsize.x/2,pos.y,unitsize.x,unitsize.y);
        break;
    case SCOUT:
        masked_blit( graphicset->scout[owner],buffer,0,0,pos.x-unitsize.x/2,pos.y,unitsize.x,unitsize.y);
        break;
    case TERRAFORMER:
        masked_blit( graphicset->terraf[owner],buffer,0,0,pos.x-unitsize.x/2,pos.y,unitsize.x,unitsize.y);
        break;
    case MAGICIAN:
        masked_blit( graphicset->magician[owner],buffer,0,0,pos.x-unitsize.x/2,pos.y,unitsize.x,unitsize.y);
        break;
    case FOG:
        masked_blit( graphicset->fog,buffer,0,0,pos.x,pos.y,tilesize.x,tilesize.y);
        break;
    default:
        break;
    }
}

void UI_Engine::drawSelection(Coordinates pos){
    masked_blit( graphicset->marker,buffer,0,0,pos.x,pos.y,graphicset->marker->w,graphicset->marker->h);
}

void UI_Engine::lockScreen(bool doit){
    if(doit)
        acquire_bitmap(buffer);
    else
        release_bitmap(buffer);
}

Coordinates UI_Engine::getTileSize(){
    return tilesize;
}
