/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "UI_Menu.hpp"

UI_Menu::UI_Menu(Coordinates size, Coordinates pos, GraphicSet* gs, UI* par){
    init(size, pos, gs, par);
}

UI_Menu::~UI_Menu(){
    ;
}

void UI_Menu::drawBox(){
    clear_bitmap(buffer);
    rect(buffer,0,0,size.x-1,size.y-1,makecol(255,255,255));
}

void UI_Menu::drawText(std::string text, MENUITEMMODE mode, unsigned int pos){
    calcColors(mode);
    textout_centre_ex(buffer, font, text.c_str(), size.x/2, pos, fg, bg);
}

void UI_Menu::drawTextLeft(std::string text, MENUITEMMODE mode, Coordinates pos){
    calcColors(mode);
    textout_ex(buffer, font, text.c_str(), pos.x, pos.y,  fg, bg);
}

void UI_Menu::drawTextCenter(std::string text, MENUITEMMODE mode, Coordinates pos){
    calcColors(mode);
    textout_centre_ex(buffer, font, text.c_str(), pos.x, pos.y,  fg, bg);
}

void UI_Menu::drawTextRight(std::string text, MENUITEMMODE mode, Coordinates pos){
    calcColors(mode);
    textout_right_ex(buffer, font, text.c_str(), pos.x, pos.y,  fg, bg);
}

void UI_Menu::calcColors(MENUITEMMODE mode){
    switch(mode){
    case HEADLINE:
        fg = makecol(255,0,0);
        bg = -1;
        break;
    case SELECTED:
        fg = makecol(63,63,63);
        bg = makecol(255,255,255);
        break;
    case DISABLED:
        fg = makecol(127,127,127);
        bg = -1;
        break;
    default:
        fg = makecol(191,191,191);
        bg = makecol( 63, 63, 63);
        break;
    }
}
