 /* Hexwars - A Hexagonal Turn Based Strategy Game
  * Copyright (C) 2009-2011  Patrik Schönfeldt
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

#ifndef FSHANDLER_HPP
#define FSHANDLER_HPP

#include <vector>
#include <iostream>
#include <boost/filesystem.hpp>
#include <boost/filesystem/fstream.hpp>

namespace fs = boost::filesystem;

class FSHandler{
public:
    FSHandler();
    ~FSHandler();

    /** \short finds files on your filesystem
     *
     * getFillPath(fs::path) allows you to work with
     * fake-relative paths
     *
     * \returns absolute path of the (partly) described file
     */
    fs::path getFullPath(fs::path filename);

    /** \short prepares directorys for later use
     *
     * \returns nothing, but may throw exceptions
     */
    void prepareDirectory(fs::path dirname);

    /** sets exedir and adds it to searchdirs */
    void setExeDir(fs::path exename);

    /// \returns absolute path to directory of binary
    fs::path getExeDir();

    /// \returns absolute path to directory of game content
    fs::path getContDir();

private:
    fs::path searchdir;
    std::vector<fs::path> searchdirs;

    fs::path exedir;
    fs::path contdir;
};

#endif // FSHANDLER_HPP
