/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "MapLoader.hpp"

//the constructor initiates everything the MapLoader has to do
MapLoader::MapLoader(std::string levelname, FSHandler* ofsh){
    fsh = ofsh;
    player = new UnitContainer();
    loadMap("maps/" + levelname);
}

    /* HWmap_V01 format saves owner of a unit in
     * the field before the last digit of a decimal
     * number. */
int MapLoader::owner(int feld){
    return ((feld%100 - feld%10)/10);
}
    /* HWmap_V01 format saves unit (and terrain!)
     * in the last digit of a decimal number. */
int MapLoader::unit(int feld){
    return (feld%10);
}

//loads the map
void MapLoader::loadMap(fs::path path){
    fs::ifstream mapload;
    mapload.open(fsh->getFullPath(path));
    char Version[10];
    for (int i=0;i<9;i++){
        mapload >> Version[i];
    }
    Version[9] = '\0';
        /* check version string of the map; later versions
         * of hexwars may support greater levels, more units. */
    if (std::string(Version) == "HWmap_V01"){

        //load the size of the map
        int X, Y;
        mapload >> X;
        mapload >> Y;

        map = new Map(X,Y,1);
        int tmpmap[32][32];
        //load all data into the the 2D array
        for (int y=Y; y>0; y--){
            for (int x=0; x<X; x++) mapload >> (tmpmap[x][Y-y]);
        }

        if(!mapload.good()){
            throw Hexception("Error loading map.\n");
            mapload.close();
            return;
        }

        mapload.close();

        //move units from map into containers
        for (int x=0; x<X; x++){
            for (int y=0; y<Y; y++){
                /* HWmap_V01 only can handle Units of PLAYER1
                 * and PLAYER2. PLAYER0 is reserved for Terrain. */
                if (owner(tmpmap[x][y]) != PLAYER0){
                    switch (unit(tmpmap[x][y])){
                    case MAGICIAN:
                        player->saveUnit(new Magician(    Coordinates(x,y),
                                                          owner(tmpmap[x][y])));
                        break;
                    case SCOUT:
                        player->saveUnit(new Scout(       Coordinates(x,y),
                                                          owner(tmpmap[x][y])));
                        break;
                    case TERRAFORMER:
                        player->saveUnit(new TerraFormer( Coordinates(x,y),
                                                          owner(tmpmap[x][y])));
                        break;
                    case WARRIOR:
                        player->saveUnit(new Warrior(     Coordinates(x,y),
                                                          owner(tmpmap[x][y])));
                        break;
                    default:
                        break;
                    }
                }
                /* HWmap_V01 saves water as 0, everything else
                 * must be assumed as height 1 since units are
                 * saved in the map. */
                if (tmpmap[x][y] == 0){
                    map->setHeight(x,y,0);
                }
                else{
                    map->setHeight(x,y,1);
                }
            }
        }
        //end of transferring units into containers
    }
    //end of "HWmap_V01" handling
    else{
        throw Hexception("Unknown map format or map not found.\n");
    }

    for (unsigned int i=0; i<player->getLatestUuid(); i++){
        Unit* u = player->getUnit(i);
        if (u != NULL){
            u->setMap(map);
        }
    }
    return;
}

MapLoader::~MapLoader(){
    ;
}

Map* MapLoader::loadedMap(){
    return map;
}

UnitContainer* MapLoader::loadedUnits(){
    return player;
}
