/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* This is meant to be used to test the 
 * implementation of units. */


#include <iostream>

#include "../definitions/Coordinates.hpp"
#include "UnitContainer.hpp"
#include "../definitions/Alias.hpp"
#include "Magician.hpp"
#include "Warrior.hpp"
#include "Scout.hpp"
#include "TerraFormer.hpp"


UnitContainer* units;
UnitContainer* ntcht;

void unitInfo(UnitContainer* showme){
    for(unsigned int i=1; i<units->getLatestUuid(); i++){
        Unit* unit = showme->getUnit(i);
        if (unit != NULL){
            std::cout << i << ": " << unit->getOwner() << ", ";
            std::cout << "(" <<  unit->getPos().x
                       << "/" << unit->getPos().y << "), ";
            switch(unit->getType()){
            case WARRIOR:
                std::cout << "Wario";
                break;
            case SCOUT:
                std::cout << "Scout";
                break;
            case TERRAFORMER:
                std::cout << "Teraf";
                break;
            case MAGICIAN:
                std::cout << "Magic";
                break;
            default:
                std::cout << "none ";
                break;
            }
            std::cout << ", " << unit->getHp() << std::endl;
        }
    }
}

int main(int argc, char** argv){
    units = new UnitContainer;
    units->saveUnit(new Magician(    Coordinates(1,2), PLAYER1));
    units->saveUnit(new Warrior(     Coordinates(1,2), PLAYER1));
    units->saveUnit(new TerraFormer( Coordinates(1,2), PLAYER2));
    units->saveUnit(new Scout(       Coordinates(1,2), PLAYER1));
    units->saveUnit(new Magician(    Coordinates(1,2), PLAYER2));
    units->saveUnit(new Warrior(     Coordinates(3,5), PLAYER2));
    units->saveUnit(new Scout(       Coordinates(3,5), PLAYER1));
    units->saveUnit(new Magician(    Coordinates(3,5), PLAYER1));
    units->saveUnit(new TerraFormer( Coordinates(3,5), PLAYER1));

    (units->getUnit(3))->kill();
    units->tidy();

    ntcht = new UnitContainer(*units);

    std::cout	<< "This is a test utility for units."
                    "It will show all Units in the format "
                   "'ID: Owner, Position, Type, HitPoints'\n";

    std::cout << std::endl << "-------------------------------" << std::endl;
    std::cout << "Units that will fight:" << std::endl;
    unitInfo(units);

    std::cout << std::endl << "-------------------------------" << std::endl;
    std::cout << "Units after the fight:" << std::endl;
    units->doMovement();
    unitInfo(units);

    std::cout << std::endl << "-------------------------------" << std::endl;
    std::cout << "Copy of original Units:" << std::endl;
    unitInfo(ntcht);
}
