/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "Unit.hpp"

Unit::Unit(){
    type = NOUNIT;
    uuid = 0;
    speed = 0;
    sight = 0;
    map = NULL;
}

Unit::Unit(const Unit& orig){
    init(orig.pos, orig.owner);
    type = orig.type;
    uuid = orig.uuid;
    speed = orig.speed;
    sight = orig.sight;
    map = orig.map;
    hp = orig.hp;
    oldhp = orig.oldhp;
}

void Unit::init(Coordinates opos, PLAYER player) {
    oldhp = hp;
    pos = opos;
    owner = player;
}

unsigned char Unit::getSight(){
    return sight;
}

Coordinates Unit::getPos(){
    return pos;
}

void Unit::command(Coordinates target){
    if(target == pos){
        way.clear();
    }
    if (way.size() == 0){
        if(pos.distanceTo(target) == 1){
            way.push_back(target);
        }
    }else{
        if(target.distanceTo(way.back()) == 1){
            way.push_back(target);
        }
    }
}

bool Unit::isAlive(){
    return (hp > 0);
}

void Unit::move(unsigned int time) {
    if (way.size() > 0) {
        for (unsigned int i = 100; i <= speed*100; i+=100) {
            /* time runs from 1 to 100, the folowing
             * equation simulates constant movement
             * of all units (slow and fast ones)
             */
            if ((i / (speed + 1)) == time) {
                oldway.push_back(pos);
                pos = way.front();
                way.erase(way.begin());
                if(map->getTerrain(pos) == WATER){
                    kill();
                }
                return;
            }
        }
    }
    return;
}

short Unit::getHp() {
    return hp;
}

short Unit::getOldHp() {
    return oldhp;
}

void Unit::newRoundPoke(){
    oldhp = hp;
}

void Unit::changeHp(short val) {
    if (hp + val < 0){
        hp = 0;
    } else if (hp + val > 100){
        hp = 100;
    } else {
        hp += val;
    }
}

void Unit::kill(){
    if (hp > 0){
        hp = 0;
    } else {
        hp--;
    }
}

unsigned int Unit::getUuid(){
    return uuid;
}

PLAYER Unit::getOwner(){
    return owner;
}

void Unit::setUuid(unsigned int id){
    if(uuid == 0){
        uuid = id;
    }
}

void Unit::setMap(Map* m){
    map = m;
}

std::vector<Coordinates> Unit::getWay(){
    return way;
}

std::vector<Coordinates> Unit::getOldWay(){
    return oldway;
}

void Unit::delOldWay(){
    oldway.clear();
    oldway.push_back(pos);
}

UNITTYPE Unit::getType(){
    return type;
}

void Unit::sync(Unit* orig){
    hp = orig->hp;
    pos = orig->pos;
    way = orig->way;
    oldway = orig->oldway;
    oldhp = orig->oldhp;
}
