/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "UnitContainer.hpp"

UnitContainer::UnitContainer(){
    latestuuid = 1;
    selectedunit = -1;
    unit.resize(0);
}

UnitContainer::UnitContainer(const UnitContainer& orig){
    latestuuid = orig.latestuuid;
    selectedunit = -1;
    unit.resize(0);
    for( unsigned int i=0; i<orig.unit.size(); i++){
        saveUnit(new Unit(*(orig.unit[i])));
    }
}

UnitContainer::~UnitContainer(){
    for (unsigned int i=0; i<unit.size(); i++){
        delete unit[i];
    }
}

unsigned int UnitContainer::getLatestUuid(){
    return latestuuid;
}

Unit* UnitContainer::getUnit(unsigned int uuid){
    for(unsigned int i = 0; i<unit.size(); i++){
        if( (unit[i])->getUuid() == uuid){
            return (unit[i]);
        }
    }
    return NULL;
}

void UnitContainer::fight() {
    float p = 0.5f;
    if (unit.size() < 2)
        return;
    srand(time(NULL));
    PLAYER attacker = PLAYER1;
    while (attackCondition()) {
        attacker = attacker % 2 + 1;
        p = 1-p;
        for (unsigned int i = 0; i < unit.size(); i++) {
            if (unit[i]->isAlive() && unit[i]->getOwner() == attacker) {
                for (unsigned int j = 0; j < unit.size(); j++) {
                    if (i != j && unit[j]->isAlive() && unit[i]->isAlive()
                            && unit[i]->getOwner() != unit[j]->getOwner()
                            && (unit[i]->getPos() == unit[j]->getPos())){
                        if (unit[i]->getType() == TERRAFORMER
                            && unit[j]->getType() == WARRIOR){
                            ((TerraFormer*) unit[i])->terraForm();
                            unit[j]->kill();
                        } else if (unit[j]->getType() == TERRAFORMER
                                   && unit[i]->getType() == WARRIOR){
                            ((TerraFormer*) unit[j])->terraForm();
                            unit[i]->kill();
                        } else if ((float)(rand()) / (RAND_MAX) < p) {
                            unit[j]->changeHp(-(1+unit[i]->getHp()
                                                /unit[j]->getHp()));
                        } else {
                            unit[i]->changeHp(-(1+unit[j]->getHp()
                                                /unit[i]->getHp()));
                        }
                    }
                }
            }
        }
    }
}

bool UnitContainer::attackCondition() {
    for (unsigned int i=0; i < unit.size(); i++) {
        for (unsigned int j=0; j<i; j++){
            if (unit[i]->getOwner() != unit[j]->getOwner()
                    && unit[i]->getPos() == unit[j]->getPos()
                    && unit[j]->isAlive() && unit[i]->isAlive()){
                return true;
            }
        }
    }
    return false;
}

void UnitContainer::newRoundPoke(){
    for(unsigned int i = 0; i<unit.size(); i++){
        if (unit[i] != NULL){
            unit[i]->newRoundPoke();
        }
    }
}

void UnitContainer::doMovement(){
    newRoundPoke();
    unsigned int time;
    for(int i=0; i<5; i++){
        switch(i){
        case 0:
            time = 25;
            break;
        case 1:
            time = 33;
            break;
        case 2:
            time = 50;
            break;
        case 3:
            time = 66;
            break;
        case 4:
            time = 75;
            break;
        default:
            break;
        }
        for(unsigned int i = 0; i<unit.size(); i++){
            if (unit[i] != NULL && unit[i]->isAlive()){
                Coordinates oldpos = unit[i]->getPos();
                unit[i]->move(time);
                for(unsigned int j = i+1; j<unit.size(); j++){
                    if (unit[j] != NULL){
                        float p = 0.5f;
                        if (((unit[j]->getWay()).size()) != 0){
                            while (  (unit[j]->getWay()).front() == oldpos &&
                                     unit[i]->getOwner() != unit[j]->getOwner() &&
                                     unit[i]->getPos() == unit[j]->getPos() &&
                                     unit[j]->isAlive() && unit[i]->isAlive()){
                                if ( unit[i]->getType() == TERRAFORMER &&
                                     unit[j]->getType() == WARRIOR){
                                    ((TerraFormer*) unit[i])->terraForm();
                                    unit[j]->kill();
                                } else if( unit[j]->getType() == TERRAFORMER &&
                                           unit[i]->getType() == WARRIOR){
                                    ((TerraFormer*) unit[j])->terraForm();
                                    unit[i]->kill();
                                } else if ((float)(rand()) / (float)(RAND_MAX) < p) {
                                    unit[j]->changeHp(-(1+unit[i]->getHp()/unit[j]->getHp()));
                                } else {
                                    unit[i]->changeHp(-(1+unit[j]->getHp()/unit[i]->getHp()));
                                }
                            }
                        }
                    }
                }
            }
        }
        fight();
    }
    merge();
}

void UnitContainer::merge(){
    for(unsigned int i = 0; i<unit.size(); i++){
        for(unsigned int j = 0; j<i; j++){
            if( unit[i]->getPos() ==  unit[j]->getPos() ){
                if( unit[i]->getHp() > unit[j]->getHp()){
                    unit[i]->changeHp(unit[j]->getHp());
                    unit[j]->kill();
                } else {
                    unit[j]->changeHp(unit[i]->getHp());
                    unit[i]->kill();
                }
            }
        }
    }
}

void UnitContainer::tidy(){
    for(unsigned int i = 0; i<unit.size(); i++){
        unit[i]->delOldWay();
        if( unit[i]->getOldHp() == 0 ){
            delete unit[i];
            unit.erase(unit.begin()+i);
            i--;
        }
    }
}

void UnitContainer::fullTidy(){
    for(unsigned int i = 0; i<unit.size(); i++){
        unit[i]->delOldWay();
        if( !(unit[i]->isAlive()) ){
            delete unit[i];
            unit.erase(unit.begin()+i);
            i--;
        }
    }
}

void UnitContainer::sync(UnitContainer* orig){
    for( unsigned int i=0; i<orig->unit.size(); i++){
        if (orig->unit[i]->getUuid() > latestuuid){
            saveUnit(new Unit(*(orig->unit[i])));
        }
        Unit* tmp = getUnit(orig->unit[i]->getUuid());
        if (tmp != NULL){
            tmp->sync(orig->unit[i]);
        }
    }
}

void UnitContainer::sync(UnitContainer* orig, PLAYER plyr){
    for( unsigned int i=0; i<orig->unit.size(); i++){
        if (orig->unit[i]->getOwner() == plyr){
            if (orig->unit[i]->getUuid() > latestuuid){
                saveUnit(new Unit(*(orig->unit[i])));
            }
            Unit* tmp = getUnit(orig->unit[i]->getUuid());
            if (tmp != NULL){
                tmp->sync(orig->unit[i]);
            }
        }
    }
}


Unit* UnitContainer::getUnit(Coordinates pos){
    for(unsigned int i = 0; i<unit.size(); i++){
        if( (unit[i])->getPos() == pos){
            return (unit[i]);
        }
    }
    return NULL;
}

Unit* UnitContainer::getSelectedUnit(){
    if (selectedunit < 0){
        return NULL;
    }
    return (unit[selectedunit]);
}

void UnitContainer::saveUnit(Unit* u){
    if (u != NULL){
        if( u->getUuid() == 0){
            u->setUuid(latestuuid++);
        } else {
            for( unsigned int i = 0; i<unit.size(); i++){
                if( (unit[i])->getUuid() == u->getUuid()){
                    return;
                }
            }
        }
        unit.push_back(u);
    }
}


bool UnitContainer::checkForUnit(UNITTYPE type, PLAYER p){
    for (unsigned int i=0; i<unit.size(); i++){
        if( ( type == NOUNIT or unit[i]->getType() == type)
            and unit[i]->getOwner() == p and unit[i]->isAlive()){
            return true;
        }
    }
    return false;
}

Unit* UnitContainer::selectNextUnit(UNITTYPE type, PLAYER p){
    int i;
    if ( selectedunit+1 < (int) unit.size()){
        i = selectedunit+1;
    } else {
        i = 0;
    }
    while(i != selectedunit){
        if( (type == NOUNIT or unit[i]->getType() == type)
            and unit[i]->getOwner() == p) {
            selectedunit = i;
            return (unit[i]);
        }
        if ( i+1 < (int) unit.size()){
            i++;
        } else {
            i = 0;
        }
    }
    if( (type == NOUNIT or unit[i]->getType() == type)
        and unit[i]->getOwner() == p) {
        selectedunit = i;
        return (unit[i]);
    }
    return NULL;
}

Unit* UnitContainer::selectPrevUnit(UNITTYPE type, PLAYER p){
    int i;
    if ( selectedunit == 0){
        i = (int) unit.size() -1;
    } else {
        i = selectedunit-1;
    }
    while(i != selectedunit){
        if( (type == NOUNIT or unit[i]->getType() == type)
            and unit[i]->getOwner() == p) {
            selectedunit = i;
            return (unit[i]);
        }
        if ( i > 0){
            i--;
        } else {
            i = (int) unit.size() -1;
        }
    }
    if( (type == NOUNIT or unit[i]->getType() == type)
        and unit[i]->getOwner() == p) {
        selectedunit = i;
        return (unit[i]);
    }
    return NULL;
}

Unit* UnitContainer::selectUnit(Coordinates pos){
    for(unsigned int i = 0; i<unit.size(); i++){
        if( (unit[i])->getPos() == pos){
            selectedunit = i;
            return (unit[i]);
        }
    }
    selectedunit = -1;
    return NULL;
}
