/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/** The class 'UnitContainer' is designed to store
 *  all the units a Client (or the Server) knows.
 *  It may (and will!) contain outdated data, because
 *  it is only refeshed based on the field of view.
 */

#ifndef _PLAYERDATA_H
#define	_PLAYERDATA_H

#include <vector>

#include "../definitions/Alias.hpp"
#include "Unit.hpp"
#include "TerraFormer.hpp"

class UnitContainer{
public:
    UnitContainer();
    UnitContainer(const UnitContainer& orig);
    ~UnitContainer();

    unsigned int getLatestUuid();

    /** returns the unit with the specified uuid.
     */
    Unit* getUnit(unsigned int uuid);

    /** indeet, movement is currently calculated
     *  by PlayerData, will be moved to Server
     *  as soon as it is ready for that.
     */
    void doMovement();

    /** @return pointer to the unit at the
     *  specified position on the Map
     */
    Unit* getUnit(Coordinates pos);

    /** @return pointer to the currently selected
     *  Unit (as saved in PlayerData, may be other
     *  selection than elsewhere)
     */
    Unit* getSelectedUnit();

    /** saves the given Unit, will handle it as
     *  a new one if it has uuid 0
     *  or a not yet used uuid.
     */
    void saveUnit(Unit* unit);

    /** @return true if Unit exists,
     * false if it does not */
    bool checkForUnit(UNITTYPE, PLAYER);

    /** selects next Unit of given UNITTYPE that is
     *  owned by given PLAYER.
     *
     *  @return a pointer to that Unit (NULL if it does
     *  not exist)
     */
    Unit* selectNextUnit(UNITTYPE, PLAYER);
    Unit* selectPrevUnit(UNITTYPE, PLAYER);

    /** selects the unit at the given position
     *
     *  @return a pointer to that Unit (NULL if it does
     *  not exist)
     */
    Unit* selectUnit(Coordinates pos);

    /** deletes units with oldhp == 0 */
    void tidy();

    /** deletes dead units */
    void fullTidy();

    void sync(UnitContainer* orig);
    void sync(UnitContainer* orig, PLAYER plyr);
private:
    /** stores all units of a Player
     */
    std::vector<Unit*> unit;

    /** the latest uuid that was given,
     *  it is used to give unique IDs to all Units
     */
    unsigned int latestuuid;

    /** the position of the currently
     *  selected unit in the vector
     */
    int selectedunit;

    /** checkes if a fight is going on
     */
    bool attackCondition();

    void merge();

    void newRoundPoke();;

    /** this does all the voodoo to reduce
     *  Units' HP
     */
    void fight();
};

#endif	/* _PLAYERDATA_H */
